package Video.Integer_Geometry with Pure is

	subtype Coordinate is Integer;

	type Coordinate_Product is range -(2 * Coordinate'First**2) .. 2 * Coordinate'Last**2;
	-- An extended range type for geometry computations

	subtype Distance is Coordinate'Base range 0 .. (
		if Coordinate'Last > Coordinate'Base'Last + Coordinate'First then
			Coordinate'Base'Last else Coordinate'Last - Coordinate'First);

	type Point is record
		X, Y : Coordinate;
	end record;
	-- A 2D vector

	-- Note: As in many other graphics systems integral coordinate points
	-- between pixels. Pixel center is 0.5 away from it.
	--    Y
	--    ^
	--    |
	--  3 +--
	--    | * |   __ Pixel center at (1.5, 1.5)
	--  2 +---+  /
	--    | * | *
	--  1 +---+---+-
	--    | * | * | * |
	--  0 +---+---+---+--> X
	--    0   1   2   3
	-- When dealing with rasters and their index ranges offset by -1 appear at the Last index.

	-- Note on simple hairline graphics: It generally takes integer coordinates
	-- and use them to address adjacent pixel usually centered at (+0.5, +0.5).
	-- Clipping is (believed to be) performed correctly against pixel edges.

	function "+" (Q : Point) return Point is (Q);

	function "-" (Q : Point) return Point is (X => -Q.X, Y => -Q.Y);

	function "+" (P, Q : Point) return Point is (X => P.X + Q.X, Y => P.Y + Q.Y);

	function "-" (P, Q : Point) return Point is (X => P.X - Q.X, Y => P.Y - Q.Y);

	function "*" (K : Integer; P : Point) return Point is (X => K * P.X, Y => K * P.Y);

	function "*" (P : Point; K : Integer) return Point is (X => P.X * K, Y => P.Y * K);

	function "/" (P : Point; D : Integer) return Point is (X => P.X / D, Y => P.Y / D);
	--

	type Interval is record
		First, Last : Coordinate;
	end record;
	-- A type like an Ada range (but a type)

	function Empty_Interval return Interval is (Coordinate'Last, Coordinate'First);

	function Is_Empty (A : Interval) return Boolean is (A.First >= A.Last);

	function Length (A : Interval) return Distance is
	(if Is_Empty (A) then 0 else A.Last - A.First);

	function Contains (A : Interval; X : Coordinate) return Boolean is (X in A.First .. A.Last);

	function Center (A : Interval) return Coordinate
	is (A.First / 2 + A.Last / 2 + (A.First rem 2 + A.Last rem 2) / 2);

	function "+" (A : Interval; B : Coordinate) return Interval is
	(A.First + B, A.Last + B);
	-- Translate interval

	function "and" (A, B : Interval) return Interval is
	(Coordinate'Max (A.First, B.First), Coordinate'Min (A.Last, B.Last));
	-- Intersection

	function "or" (A, B : Interval) return Interval is
	(if Is_Empty (A) then B
	elsif Is_Empty (B) then A
	else (Coordinate'Min (A.First, B.First), Coordinate'Max (A.Last, B.Last)));
	-- Minimal interval enclosing both arguments

	type Box is record
		X, Y : Interval;
	end record;

	function Empty_Box return Box is (Empty_Interval, Empty_Interval);

	function Is_Empty (B : Box) return Boolean is (Is_Empty (B.X) or else Is_Empty (B.Y));

	function "+" (P : Box; Q : Point) return Box is
	(P.X + Q.X, P.Y + Q.Y);
	-- Translate box

	function "and" (P, Q : Box) return Box is
	(P.X and Q.X, P.Y and Q.Y);
	-- Intersection

	function "or" (P, Q : Box) return Box is
	(P.X or Q.X, P.Y or Q.Y);
	-- Minimal box enclosing both arguments

	function Center (P : Box) return Point
	is (X => Center (P.X), Y => Center (P.Y));
end Video.Integer_Geometry;